<?php

try {
    require_once(dirname(__FILE__) . '/application.php');
    require_once(dirname(__FILE__) . '/setup.php');
    require_once(dirname(__FILE__) . '/lib/import.php');
    require_once(dirname(__FILE__) . '/lib/schema.php');

    define('ERROR_INVALID_STEP_MSG', 'Invalid step is selected. Please initialize system, and retry installation.');

    class Install_Factory extends Setup
    {
        public function __construct()
        {
            $this->initialize();
            $_SESSION = array();

            switch ( $this->getPhase() ) {
                case '1'    :
                    if ( Setup_Detector::IsInstalled() ) die('system has already installed.');
                    $Step = new Install_Step_1($this->Post);
                    break;
                case '2'    :
                    $Step = new Install_Step_2($this->Post);
                    break;
                case '3'    :
                    include(dirname(__FILE__) . '/lib/db_default.php');
                    $dbHost = $this->Post->get('db_host');
                    $dbName = $this->Post->get('db_name');
                    $dbCreate = $this->Post->get('db_create');
                    $dbUser = $this->Post->get('db_user');
                    $dbPass = $this->Post->get('db_pass');
                    $dbPrefix = $this->Post->get('db_prefix');
                    $dbCharset = $this->Post->get('db_charset');

                    if ( empty($dbHost) && isset($dbDefaultHost) ) {
                        $this->Post->set('db_host', $dbDefaultHost);
                        $_POST['db_host'] = $dbDefaultHost;
                    }
                    if ( empty($dbCreate) && isset($dbDefaultCreate)) {
                        $this->Post->set('db_create', $dbDefaultCreate);
                        $_POST['db_create'] = $dbDefaultCreate;
                    }
                    if ( empty($dbName) && isset($dbDefaultName) ) {
                        $this->Post->set('db_name', $dbDefaultName);
                        $_POST['db_name'] = $dbDefaultName;
                    }
                    if ( empty($dbUser) && isset($dbDefaultUser) ) {
                        $this->Post->set('db_user', $dbDefaultUser);
                        $_POST['db_user'] = $dbDefaultUser;
                    }
                    if ( empty($dbPass) && isset($dbDefaultPass) ) {
                        $this->Post->set('db_pass', $dbDefaultPass);
                        $_POST['db_pass'] = $dbDefaultPass;
                    }
                    if ( empty($dbPrefix) && isset($dbDefaultPrefix) ) {
                        $this->Post->set('db_prefix', $dbDefaultPrefix);
                        $_POST['db_prefix'] = $dbDefaultPrefix;
                    }
                    if ( empty($dbCharset) && isset($dbDefaultCharset) ) {
                        $this->Post->set('db_charset', $dbDefaultCharset);
                        $_POST['db_charset'] = $dbDefaultCharset;
                    }
                    $Step = new Install_Step_3($this->Post);
                    break;
                case '4'    :
                    $Step = new Install_Step_4($this->Post);
                    break;
                case '5'    :
                    $Step = new Install_Step_5($this->Post);
                    break;
                case '6'    :
                    $Step = new Install_Step_6($this->Post);
                    break;
                default     :
                    die(ERROR_INVALID_STEP_MSG);
                    break;
            }

            $flg = $this->Post->get('do');
            if ( $Step->validate() && !empty($flg) ) $Step->business();
            print SetupCommon::getTpl($Step->results());
            Cache::allFlush();
        }

        function getPhase()
        {
            return $this->Post->get('phase', '1');
        }
    }

    class Install_Step
    {
        var $Post;  // is_object
        var $Error; // is_array
        var $Notice;// is_array
        var $Trans; // is_array
        var $Step;  // is_string

        /*
        abstract function validate();
        abstract function business();
        */

        function __construct($post)
        {
            $this->Tpl = SetupCommon::setTpl(dirname(__FILE__) . '/tpl/install.html');
            $this->Post = $post;
        }

        function fileExists($name, $path)
        {
            if ( !file_exists($path) ) $this->Error[$name] = true;
        }

        function checkPermission($name, $path)
        {
            if ( !SetupCommon::checkPermission($path) ) $this->Error[$name] = true;
        }

        function buildError()
        {
            if ( !empty($this->Error) ) {
                $root = array($this->Step, $this->Post->get('phase'));
                foreach ( $this->Error as $key => $val ) {
                    if ( is_array($val) ) {
                        foreach ( $val as $k => $v ) {
                            $loop = array_merge(array($key . ':loop'), $root);
                            $this->Tpl->add($loop, array($key => $k));
                        }
                    }
                    $error = array_merge(array($key . ':error'), $root);
                    $this->Tpl->add($error);
                }
                $this->Trans = $_POST;
                return true;
            } else {
                return false;
            }
        }

        function buildNotice()
        {
            if ( !empty($this->Notice) ) {
                $root = array($this->Step, $this->Post->get('phase'));
                foreach ( $this->Notice as $key => $val ) {
                    if ( is_array($val) ) {
                        foreach ( $val as $k => $v ) {
                            $loop = array_merge(array($key . ':loop'), $root);
                            $this->Tpl->add($loop, array($key => $k));
                        }
                    }
                    $notice = array_merge(array($key . ':notice'), $root);
                    $this->Tpl->add($notice);
                }
                return true;
            } else {
                return false;
            }
        }

        function results($vars = null)
        {
            $flg = $this->Post->get('do');
            $create = $this->Post->get('db_create', false);

            if ( !empty($flg) ) {
                if ( !empty($this->Error) ) {
                    $this->Step = 'step#error';
                    $this->buildError();
                } else {
                    $this->Step = 'step#result';
                    $this->buildNotice();
                }
            } else {
                $this->Step = 'step#apply';
                $this->Trans = $_POST;
            }

            $step = $this->Step;
            $phase = $this->Post->get('phase', '1');
            include(dirname(__FILE__) . '/lib/db_default.php');
            if ($create === 'checked') {
                $this->Tpl->add(array('db_create:touch#true', $step, $phase));
            }

            if (is_array($this->Trans)) {
                foreach ($this->Trans as $key => $val) {
                    $this->Trans[$key . ':checked#' . $val] = ' checked';
                    $this->Trans[$key . ':selected#' . $val] = ' selected';
                }
            }
            $this->Tpl->add(array($step, $phase), $this->Trans);
            $this->Tpl->add($phase);

            return $this->Tpl;
        }

        function writeConfig($key, $val)
        {
            $val = str_replace('$', '\$', $val);
            $pattern = '/define\(\'' . $key . '\'\s*,\s*\'\S*\'\s*\);/';
            $replace = "define('$key', '$val');";

            $config = file_get_contents(PATH_CONFIG);
            $config = preg_replace($pattern, $replace, $config);

            $file = fopen(PATH_CONFIG, 'w+');
            if ( empty($file) ) $this->Error['config_open'];

            $write = fwrite($file, $config);
            if ( empty($write) ) $this->Error['config_write'];

            $close = fclose($file);
            if ( empty($close) ) $this->Error['config_close'];
        }

        function checkInput($except = array())
        {
            $fds = $this->Post->listFields();
            foreach ( $fds as $fd ) {
                $val = $this->Post->get($fd, null);
                if ($fd === 'db_pass') {
                    if (empty($val)) {
                        continue;
                    }
                    $val = htmlspecialchars_decode($val, ENT_QUOTES);
                    $pattern = '/^[0-9a-zA-Z' . preg_quote('!#%&()*+,-./;<=>?@[]^_{|}~', '/') . ']+$/';
                    if (!preg_match($pattern, $val)) {
                        $this->Error[$fd . '_input'] = true;
                    }
                } elseif (!empty($val) || $fd === 'db_create') {
                    continue;
                } else {
                    $this->Error[$fd . '_input'] = true;
                }
            }
        }
    }

    class Install_Step_1 extends Install_Step
    {
        function validate()
        {
            // comparing PHP
            if ( version_compare(PHP_VERSION, '5.3.3', '<') ) {
                $this->Error['php_version'] = true;
            }

            // GD
            if ( !function_exists('gd_info') ) {
                $this->Error['gd_exists'] = true;
            }
            // SPL
            if ( !function_exists('spl_autoload_register') ) {
                $this->Error['spl_exists'] = true;
            }
            // hash_hmac
            if ( !function_exists('hash_hmac') ) {
                $this->Notice['hash_exists'] = true;
            }
            // imagerotate
            if ( !function_exists('imagerotate') ) {
                $this->Notice['imagerotate_exists'] = true;
            }
            // simplexml
            if ( !function_exists('simplexml_load_string') ) {
                $this->Notice['simplexml_exists'] = true;
            }

            // exists
            $this->fileExists('config_exists', PATH_CONFIG);
            $this->fileExists('license_exists', PATH_LICENSE);
            $this->fileExists('archives_exists', PATH_ARCHIVES);
            $this->fileExists('cache_exists', PATH_CACHE);
            $this->fileExists('themes_exists', PATH_THEMES);
            $this->fileExists('htaccess_exists', PATH_HTACCESS);

            // permission
            if ( !SetupCommon::checkPermission(PATH_ARCHIVES) ) $this->Error['archives_permission'] = true;
            if ( !SetupCommon::checkPermission(PATH_CACHE) ) $this->Error['cache_permission'] = true;
            if ( !SetupCommon::checkPermission(PATH_CONFIG, '666') ) $this->Error['config_permission'] = true;

            return empty($this->Error) ? true : false;
        }

        function business()
        {
            return empty($this->Error) ? true : false;
        }
    }

    class Install_Step_2 extends Install_Step
    {
        function validate()
        {
            if ( !$this->Post->get('domain') ) {
                $this->Error['domain_input'] = true;
            } elseif ( !preg_match('@^([a-zA-Z0-9]+[a-zA-Z0-9-]*(?<=[a-zA-Z0-9])\.)+[a-zA-Z]+[a-zA-Z0-9-]*(?<=[a-zA-Z0-9])\.?$|^(?:\d{1,3}\.){3}\d{1,3}$|^localhost$@', $this->Post->get('domain')) ) {
                $this->Error['domain_invalid'] = true;
            }

            return empty($this->Error) ? true : false;
        }

        function business()
        {
            $domain = $this->Post->get('domain');
            $this->writeConfig('DOMAIN', $domain);

            $this->Trans = $_POST;
            return empty($this->Error) ? true : false;
        }
    }

    class Install_Step_3 extends Install_Step
    {
        function validate()
        {
            $except = array('setup', 'db_pass');
            $this->checkInput($except);
            $dsn = array(
                'host' => $this->Post->get('db_host'),
                'user' => $this->Post->get('db_user'),
                'pass' => $this->Post->get('db_pass'),
                'name' => $this->Post->get('db_name'),
            );
            $DB = new Acms\Services\Database\Engine\PdoEngine;

            if ( empty($this->Error) ) {
                if (!$DB->checkConnection(dsn($dsn))) {
                    $this->Error['server_connect'] = true;
                    return false;
                }
                if (!$this->Post->get('db_create') && !$DB->checkConnectDatabase(dsn($dsn))) {
                    $this->Error['database_connect'] = true;
                    return false;
                }

                try {
                    if (!!$this->Post->get('db_create') && !!$this->Post->isExists('db_charset')) {
                        $dsn['name'] = '';
                        $DB->connect(dsn($dsn));
                        $charset = $this->Post->get('db_charset');

                        if ( preg_match('@^[shiftj]+$@i', $charset) ) {
                            $names = 'sjis';
                            $collate = 'sjis_japanese_ci';
                        } else if ( preg_match('@^[eucjp_\-]+$@i', $charset) ) {
                            $names = 'ujis';
                            $collate = 'ujis_japanese_ci';
                        } else {
                            $names = 'utf8';
                            $collate = 'utf8_general_ci';
                        }
                        $DB->query('CREATE DATABASE `' . $this->Post->get('db_name') . '` DEFAULT CHARSET=' . $names . ' COLLATE=' . $collate, 'exec');
                    }
                } catch ( \Exception $e ) {
                    $this->Error['create_database'] = true;
                    return false;
                }
                return true;
            }
            return false;
        }

        function business()
        {
            $this->writeConfig('DB_HOST', $this->Post->get('db_host'));
            $this->writeConfig('DB_NAME', $this->Post->get('db_name'));
            $this->writeConfig('DB_USER', $this->Post->get('db_user'));
            $this->writeConfig('DB_PASS', htmlspecialchars_decode($this->Post->get('db_pass'), ENT_QUOTES));
            $this->writeConfig('DB_PREFIX', $this->Post->get('db_prefix'));
            $this->writeConfig('DB_CHARSET', $this->Post->get('db_charset'));

            $this->Trans = $_POST;
            return empty($this->Error) ? true : false;
        }
    }

    class Install_Step_4 extends Install_Step
    {
        function validate()
        {
            //if ( !SetupCommon::checkPrivileges('CREATE') )   $this->Error['create_privilege']= true;
            //if ( !SetupCommon::checkPrivileges('ALTER') )    $this->Error['alter_privilege'] = true;
            //if ( !SetupCommon::checkPrivileges('SELECT') )   $this->Error['select_privilege']= true;
            //if ( !SetupCommon::checkPrivileges('UPDATE') )   $this->Error['update_privilege']= true;
            //if ( !SetupCommon::checkPrivileges('DELETE') )   $this->Error['delete_privilege']= true;
            //if ( !SetupCommon::checkPrivileges('INSERT') )   $this->Error['insert_privilege']= true;

            return empty($this->Error) ? true : false;
        }

        function business()
        {
            $DB = DB::singleton(SetupCommon::dsn());

            $Schema = new Schema(SetupCommon::dsn(), DB_NAME, DB_PREFIX);
            $Schema->defSetYaml('schema');

            if ( !!$Schema->existsDefinedTable() ) {
                /**
                 * Compare Tables
                 */
                $diff = $Schema->compareTables();
                if ( !empty($diff) ) {
                    foreach ( $diff as $tb ) {
                        $this->Error['exists_table'][$tb] = true;
                    }
                }

                /**
                 * Compare Columns
                 */
                $tbs = Schema::listUp($Schema->schema);
                foreach ( $tbs as $tb ) {
                    $Schema->compareColumns($tb);
                }

                if ( !empty($Schema->_addRAM) ) {
                    foreach ( $Schema->_addRAM as $col ) {
                        $this->Error['shortage_column'][$col] = true;
                    }
                }

                if ( !empty($Schema->_changeRAM) ) {
                    foreach ( $Schema->_changeRAM as $col ) {
                        $this->Error['disparity_column'][$col] = true;
                    }
                }

                /**
                 * Finally Detect Error
                 */
                if ( empty($this->Error) ) $this->Notice['exists_table'] = true;

            } else {
                $tbs = Schema::listUp($Schema->define);
                $fixedIndex = $Schema->defLoadYaml('index');
                $Schema->createTables($tbs, $fixedIndex);

                foreach ( $tbs as $tb ) {
                    $this->Notice['create_table'][$tb] = true;
                }
            }

            if ( !SetupCommon::checkAlterPerm() ) {
                $this->Notice['alter_privilege'] = true;
                $_POST['mysqlErr'] = DB::errorCode() . ': ' . implode(', ', DB::errorInfo());
            }

            return empty($this->Error) ? true : false;
        }
    }

    class Install_Step_5 extends Install_Step
    {
        function validate()
        {
            $except = array('setup');
            $this->checkInput($except);

            $code = $this->Post->get('user_code');
            $mail = $this->Post->get('user_mail');
            $pass = $this->Post->get('user_pass');

            if ( !preg_match(REGEX_VALID_ID, $code) ) {
                $this->Error['user_code_invalid'] = true;
            }
            if ( !preg_match(REGEX_VALID_MAIL, $mail) ) {
                $this->Error['user_mail_invalid'] = true;
            }
            if ( !preg_match(REGEX_VALID_PASSWD, $pass) ) {
                $this->Error['user_pass_invalid'] = true;
            }

            return empty($this->Error) ? true : false;
        }

        function business()
        {
            $theme = $this->Post->get('select_theme');
            $_path = dirname(__FILE__) . '/bin/';

            $parsed = SetupCommon::themeParser($theme);

            if ( empty($parsed['key']) ) {
                $theme = $parsed['val'];
                $children = $parsed['key'];
            } else {
                $theme = $parsed['key'];
                $children = $parsed['val'];
            }

            $blog = array(
                'id' => 1,
                'name' => $this->Post->get('blog_name'),
                'code' => '',
                'parent' => 0,
                'domain' => DOMAIN,
            );
            $user = array(
                'code' => $this->Post->get('user_code'),
                'name' => $this->Post->get('user_name'),
                'pass' => $this->Post->get('user_pass'),
                'mail' => $this->Post->get('user_mail'),
            );
            $path = array(
                'yaml' => $_path . $theme . '/' . $theme . '.yaml',
                'themes' => $_path . $theme . '/themes/',
                'archives' => $_path . $theme . '/archives/',
                'media' => $_path . $theme . '/media/',
                'storage' => $_path . $theme . '/storage/',
            );

            // setup directory
            $_POST['setupdir'] = 'setup_' . substr(str_shuffle('1234567890abcdefghijklmnopqrstuvwxyz'), 0, 6);

            if ( file_exists($path['yaml']) ) {

                if ( empty($children) ) {
                    /**
                     * Install Base Theme
                     */
                    new Theme_Install($path, $blog, $user);

                    /**
                     * init ENTRY_HASH
                     */
                    SetupCommon::initEntryHash(1);
                } else {
                    /**
                     * Install Base Theme
                     */
                    $static = true;
                    $Inst = new Theme_Install($path, $blog, $user, $static);

                    /**
                     * init ENTRY_HASH
                     */
                    SetupCommon::initEntryHash(1);

                    /**
                     * boot install children recursivity
                     */
                    $pram = $this->getParentRAM($Inst);
                    $this->recursiveInstall($children, 1);
                }

                /**
                 * Attachment Themes
                 */
                $attaches = array();
                if ( $this->Post->isExists('with_keitai') )
                    $attaches[] = 'keitai';
                if ( $this->Post->isExists('with_jquerymobile') )
                    $attaches[] = 'jquerymobile';

                foreach ( $attaches as $attach ) {
                    $path = array(
                        'yaml' => $_path . $attach . '/' . $attach . '.yaml',
                        'themes' => $_path . $attach . '/themes/',
                        'archives' => $_path . $attach . '/archives/',
                    );
                    if ( file_exists($path['yaml']) ) {
                        new Theme_Install_Attach($path);
                    }
                }

            } else {
                $this->Error['yaml_exist'] = true;
            }

            $this->Trans = $_POST;
            return empty($this->Error) ? true : false;
        }

        function recursiveInstall($children, $pid, $pram = null)
        {
            foreach ( $children as $key => $val ) {
                if ( is_array($val) ) {
                    $theme = $key;
                    $bid = $this->addInstall($theme, $pid, $pram);
                    $_pram = $this->getParentRAM($Inst);

                    $this->recursiveInstall($val, $bid, $_pram);
                } else {
                    $theme = $val;
                    $bid = $this->addInstall($theme, $pid, $pram);
                }
            }
        }

        function addInstall($theme, $pid, $pram = null)
        {
            $DB = DB::singleton(SetupCommon::dsn());
            $_path = dirname(__FILE__) . '/bin/';

            $blog = array(
                'id' => $DB->query(SQL::nextval('blog_id', SetupCommon::dsn()), 'seq'),
                'name' => $theme,
                'code' => $theme,
                'parent' => $pid,
                'domain' => DOMAIN,
            );

            $path = array(
                'yaml' => $_path . $theme . '/' . $theme . '.yaml',
                'themes' => $_path . $theme . '/themes/',
                'archives' => $_path . $theme . '/archives/',
                'media' => $_path . $theme . '/media/',
                'storage' => $_path . $theme . '/storage/',
            );

            $Inst = new Theme_Install_Child($path, $blog, $pram);
            SetupCommon::initEntryHash($Inst->born);

            return $Inst->born;
        }

        function getParentRAM($Inst)
        {
            $pram = array();

            $pram['category'] = $Inst->_categoryRAM;
            $pram['entry'] = $Inst->_entryRAM;
            $pram['rule'] = $Inst->_ruleRAM;
            $pram['module'] = $Inst->_moduleRAM;
            $pram['column'] = $Inst->_columnRAM;
            $pram['comment'] = $Inst->_commentRAM;
            $pram['user'] = $Inst->_userRAM;
            $pram['form'] = $Inst->_formRAM;

            return $pram;
        }
    }

    class Install_Step_6 extends Install_Step
    {
        function validate()
        {
            $new_dir_name = $this->Post->get('setup_dir_name');
            $root_dir = dirname(dirname(__FILE__)) . '/';
            if ( @rename($root_dir . 'setup', $root_dir . $new_dir_name) and $new_dir_name !== '' ) {
                die(header('Location: ' . HTTP_ROOT . LOGIN_SEGMENT . '/'));
            }

            $this->Error['error_rename_setup_dir'] = true;
            return false;
        }
    }

    class Theme_Install extends Import
    {
        function __construct($path, $blog, $user, $static = null)
        {
            // when multiblog install root blog has detected bid record is static.
            if ( !empty($static) ) {
                $this->staticBID = true;
            }
            $array = Config::yamlLoad($path['yaml']);

            $this->initSeq();

            $this->generateBlog($blog);
            $this->generateUser($user);

            if (isset($array['config_set'])) {
                $this->insertConfigSet($array['config_set']);
            }
            if (isset($array['media'])) {
                $this->insertMedia($array['media']);
            }
            if (isset($array['media_tag'])) {
                $this->insertMediaTag($array['media_tag']);
            }
            $this->insertCategory($array['category']);
            $this->insertEntry($array['entry']);
            $this->insertColumn($array['column']);
            $this->fixPrimaryImage();
            $this->insertTag($array['tag']);
            $this->insertComment($array['comment']);

            $this->insertSchedule(@$array['schedule']);

            $this->insertModule($array['module']);
            $this->insertRule($array['rule']);
            $this->insertConfig($array['config']);
            $this->insertLayout($array['layout_grid']);

            $this->insertForm($array['form']);
            $this->insertDashboard($array['dashboard']);
            $this->insertField($array['field']);

            $this->copyArchives($path['archives'], ROOT_DIR . 'archives/');
            $this->copyArchives($path['media'], ROOT_DIR . 'media/');
            $this->copyArchives($path['storage'], ROOT_DIR . 'storage/');
            $this->copyThemes($path['themes'], ROOT_DIR . 'themes/');
            $this->generateFulltext($this->BID);
        }
    }

    class Theme_Install_Attach extends Import
    {
        function __construct($path)
        {
            $array = Config::yamlLoad($path['yaml']);

            $this->initSeq();

            $this->BID = 1;
            $this->UID = 1;

            $this->insertCategory($array['category']);
            $this->insertEntry($array['entry']);
            $this->insertColumn($array['column']);
            $this->fixPrimaryImage();
            $this->insertTag($array['tag']);
            $this->insertComment($array['comment']);

            $this->insertSchedule(@$array['schedule']);

            $this->insertModule($array['module']);
            $this->insertRule($array['rule']);
            $this->insertConfig($array['config']);
            $this->insertLayout($array['layout_grid']);

            $this->insertForm($array['form']);
            $this->insertDashboard($array['dashboard']);
            $this->insertField($array['field']);

            $this->copyArchives($path['archives'], ROOT_DIR . 'archives/');
            $this->copyThemes($path['themes'], ROOT_DIR . 'themes/');
            $this->generateFulltext($this->BID);
        }
    }

    class Theme_Install_Child extends Import
    {
        function __construct($path, $blog, $pram = null)
        {
            // throw parent blog's id RAMs
            if ( !empty($pram) ) {
                $this->parentRAM = $pram;
                $this->staticBID = true;
            }
            $array = Config::yamlLoad($path['yaml']);

            $this->initSeq();

            $this->born = $this->generateBlog($blog);
            $this->UID = 1;

            $this->insertCategory($array['category']);
            $this->insertEntry($array['entry']);
            $this->insertColumn($array['column']);
            $this->fixPrimaryImage();
            $this->insertTag($array['tag']);
            $this->insertComment($array['comment']);

            $this->insertSchedule($array['schedule']);

            $this->insertModule($array['module']);
            $this->insertRule($array['rule']);
            $this->insertConfig($array['config']);
            $this->insertLayout($array['layout_grid']);

            $this->insertForm($array['form']);
            $this->insertDashboard($array['dashboard']);
            $this->insertField($array['field']);

            $this->copyArchives($path['archives'], ROOT_DIR . 'archives/');
            $this->copyThemes($path['themes'], ROOT_DIR . 'themes/');
            $this->generateFulltext($this->BID);
        }
    }

    new Install_Factory();

    App::checkException();
} catch ( Exception $e ) {
    App::showError($e);
}
